aboutsummaryrefslogtreecommitdiff
path: root/src/nvim/lua/executor.c
diff options
context:
space:
mode:
authorbfredl <bjorn.linse@gmail.com>2025-03-05 10:57:02 +0100
committerbfredl <bjorn.linse@gmail.com>2025-03-05 12:09:57 +0100
commitebb963a4a08283227233c417fed7d822c86a8807 (patch)
tree0e7f7b3027a9755e6c1a019981b13a652494472f /src/nvim/lua/executor.c
parente4c094a84da8f417687d2730a4793dd4b408739d (diff)
downloadrneovim-ebb963a4a08283227233c417fed7d822c86a8807.tar.gz
rneovim-ebb963a4a08283227233c417fed7d822c86a8807.tar.bz2
rneovim-ebb963a4a08283227233c417fed7d822c86a8807.zip
fix(lua): format errors from luv callbacks using __tostring
Diffstat (limited to 'src/nvim/lua/executor.c')
-rw-r--r--src/nvim/lua/executor.c41
1 files changed, 25 insertions, 16 deletions
diff --git a/src/nvim/lua/executor.c b/src/nvim/lua/executor.c
index b1766db379..91ff285046 100644
--- a/src/nvim/lua/executor.c
+++ b/src/nvim/lua/executor.c
@@ -125,30 +125,37 @@ lua_State *get_global_lstate(void)
return global_lstate;
}
-/// Convert lua error into a Vim error message
+/// get error on top of stack as a string
///
-/// @param lstate Lua interpreter state.
-/// @param[in] msg Message base, must contain one `%.*s`.
-void nlua_error(lua_State *const lstate, const char *const msg)
- FUNC_ATTR_NONNULL_ALL
+/// Might alter the top value on stack in place (but doesn't change stack height)
+///
+/// "error" points to memory on the lua stack, use
+/// or duplicate the string before using "lstate" again
+///
+/// @param[out] len length of error (can be NULL)
+static const char *nlua_get_error(lua_State *lstate, size_t *len)
{
- size_t len;
- const char *str = NULL;
-
if (luaL_getmetafield(lstate, -1, "__tostring")) {
if (lua_isfunction(lstate, -1) && luaL_callmeta(lstate, -2, "__tostring")) {
- // call __tostring, convert the result and pop result.
- str = lua_tolstring(lstate, -1, &len);
- lua_pop(lstate, 1);
+ // call __tostring, convert the result and replace error with it
+ lua_replace(lstate, -3);
}
// pop __tostring.
lua_pop(lstate, 1);
}
- if (!str) {
- // defer to lua default conversion, this will render tables as [NULL].
- str = lua_tolstring(lstate, -1, &len);
- }
+ return lua_tolstring(lstate, -1, len);
+}
+
+/// Convert lua error into a Vim error message
+///
+/// @param lstate Lua interpreter state.
+/// @param[in] msg Message base, must contain one `%.*s`.
+void nlua_error(lua_State *const lstate, const char *const msg)
+ FUNC_ATTR_NONNULL_ALL
+{
+ size_t len;
+ const char *str = nlua_get_error(lstate, &len);
if (in_script) {
fprintf(stderr, msg, (int)len, str);
@@ -218,7 +225,9 @@ static int nlua_fast_cfpcall(lua_State *lstate, int nargs, int nresult, int flag
// consider out of memory errors unrecoverable, just like xmalloc()
preserve_exit(e_outofmem);
}
- const char *error = lua_tostring(lstate, -1);
+
+ size_t len;
+ const char *error = nlua_get_error(lstate, &len);
multiqueue_put(main_loop.events, nlua_luv_error_event,
error != NULL ? xstrdup(error) : NULL, (void *)(intptr_t)kCallback);